﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;
using System.Runtime.InteropServices;

namespace APPRN_SamplePg
{
    public partial class frmMain : Form
    {
        private APPRNPAPI.CallBackDelegate statusCallBackDelegate = null;

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            InitControls();
            EnableControls(false);
            EnableIFControls(APPRNPAPI._IF_USB);
        }

        private void InitControls()
        {
            try{
                rdoIF_Usb.Checked               = true;
                cmbLPT_Port.SelectedIndex       = 0;
                cmbSerial_Port.SelectedIndex    = 0;
                cmbSerial_Baudrate.SelectedItem = "115200";
                cmbSerial_Databits.SelectedItem = "8";
                cmbSerial_Parity.SelectedItem   = "NONE";
                cmbSerial_Stopbits.SelectedItem = "1";
            }
            catch (System.Exception ex){
                MessageBox.Show(ex.ToString());
            }
        }

        private void EnableControls(bool bConnected)
        {
            btnConnect.Enabled      = !bConnected;
            btnDisconnect.Enabled   = bConnected;
            btnFeedCut.Enabled      = bConnected;
            btnPrintText.Enabled    = bConnected;
            btnPrintBitmap.Enabled  = bConnected;
            btn1DBarcode.Enabled    = bConnected;
            btn2DBarcode.Enabled    = bConnected;
            btnDirectIO.Enabled     = bConnected;
            btnStatus.Enabled       = bConnected;
            btnPrintSample.Enabled  = bConnected;
            btnStatus_Set.Enabled   = bConnected;
            btnStatus_Cancel.Enabled = false;
            btnPmSample.Enabled = bConnected;
            btnPmDirection.Enabled = bConnected;
        }

        private void EnableIFControls(int nInterface)
        {
            cmbLPT_Port.Enabled         = nInterface == APPRNPAPI._IF_PARALLEL;
            cmbSerial_Port.Enabled      = nInterface == APPRNPAPI._IF_SERIAL || nInterface == APPRNPAPI._IF_BLUETOOTH;
            cmbSerial_Baudrate.Enabled  = nInterface == APPRNPAPI._IF_SERIAL;
            cmbSerial_Databits.Enabled  = nInterface == APPRNPAPI._IF_SERIAL;
            cmbSerial_Parity.Enabled    = nInterface == APPRNPAPI._IF_SERIAL;
            cmbSerial_Stopbits.Enabled  = nInterface == APPRNPAPI._IF_SERIAL;
            txtNet_IPAddr.Enabled       = nInterface == APPRNPAPI._IF_LAN || nInterface == APPRNPAPI._IF_WLAN;
            txtNet_PortNum.Enabled      = nInterface == APPRNPAPI._IF_LAN || nInterface == APPRNPAPI._IF_WLAN;
        }

        private void rdoIF_Serial_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_SERIAL);
        }

        private void rdoIF_Parallel_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_PARALLEL);
        }

        private void rdoIF_Usb_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_USB);
        }

        private void rdoIF_Lan_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_LAN);
        }

        private void rdoIF_WLan_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_WLAN);
        }

        private void rdoIF_BT_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(APPRNPAPI._IF_BLUETOOTH);
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            string strPort = "";
            int nBaudrate   = 115200, nDatabits = 8, nParity = 0, nStopbits = 0, nNet_PortNum = 0;

            if (rdoIF_Serial.Checked)
            {
                strPort     = cmbSerial_Port.Text;
                nBaudrate   = Convert.ToInt32(cmbSerial_Baudrate.Text);
                nDatabits   = Convert.ToInt32(cmbSerial_Databits.Text);
                nParity     = cmbSerial_Parity.SelectedIndex;
                nStopbits   = cmbSerial_Stopbits.SelectedIndex;

                if (APPRNPAPI.PrinterOpen(APPRNPAPI._IF_SERIAL, strPort, nBaudrate, nDatabits, nParity, nStopbits) != APPRNPAPI._SUCCESS){
                    MessageBox.Show("Connect fail [Serial]");
                    return;
                }
            }
            else if(rdoIF_Parallel.Checked)
            {
                strPort = cmbLPT_Port.Text;
                if (APPRNPAPI.PrinterOpen(APPRNPAPI._IF_PARALLEL, strPort, 0, 0, 0, 0) != APPRNPAPI._SUCCESS){
                    MessageBox.Show("Connect fail [Parallel]");
                    return;
                }
            }
            else if(rdoIF_Usb.Checked)
            {
                if ( APPRNPAPI.PrinterOpen(APPRNPAPI._IF_USB, "", 0, 0, 0, 0) != APPRNPAPI._SUCCESS){
                    MessageBox.Show("Connect fail [USB]");
                    return;
                }
            }
            else if (rdoIF_Lan.Checked || rdoIF_WLan.Checked)
            {
                strPort = txtNet_IPAddr.Text;
                nNet_PortNum = Convert.ToInt32(txtNet_PortNum.Text);
                if (APPRNPAPI.PrinterOpen(rdoIF_Lan.Checked ? APPRNPAPI._IF_LAN : APPRNPAPI._IF_WLAN, strPort, nNet_PortNum, 0, 0, 0) != APPRNPAPI._SUCCESS){
                    MessageBox.Show("Connect fail [TCP/IP]");
                    return;
                }
            }
            else if (rdoIF_BT.Checked)
            {
                strPort = cmbSerial_Port.Text;
                if (APPRNPAPI.PrinterOpen(APPRNPAPI._IF_BLUETOOTH, strPort, 115200, 0, 0, 0) != APPRNPAPI._SUCCESS){
                    MessageBox.Show("Connect fail [Bluetooth]");
                    return;
                }
            }

            EnableControls(true);

            Cursor.Current = Cursors.Default;
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            APPRNPAPI.PrinterClose();
            EnableControls(false);
            statusTextBox.Clear();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            APPRNPAPI.PrinterClose();
        }

        private void btnFeedCut_Click(object sender, EventArgs e)
        {
            APPRNPAPI.CutPaper();
        }

        private void btnPrintText_Click(object sender, EventArgs e)
        {
            APPRNPAPI.InitializePrinter();
            APPRNPAPI.SetCharacterSet(APPRNPAPI._CS_WPC1252);
            APPRNPAPI.SetInterChrSet(APPRNPAPI._ICS_USA);

            // Call 'PrintText' with system default code page
            APPRNPAPI.PrintText("Left, Default\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Center, Underline\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_UNDERLINE, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Left, Underline, Bold\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_UNDERLINE | APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Left, Underline, Bold, FontB\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_UNDERLINE | APPRNPAPI._FT_BOLD | APPRNPAPI._FT_FONTB, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Left, Underline, Bold, FontC\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_UNDERLINE | APPRNPAPI._FT_BOLD | APPRNPAPI._FT_FONTC, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Right, Reverse\n", APPRNPAPI._ALIGNMENT_RIGHT, APPRNPAPI._FT_REVERSE, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Default, 0W 0H\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.PrintText("Double Height, 0W 1H\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_1WIDTH | APPRNPAPI._TS_1HEIGHT);

            //APPRNPAPI.PrintTextW("Xin chào\n\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT, APPRNPAPI._CS_TCVN);			// Vietnamese
            //APPRNPAPI.PrintTextW("안녕하세요\n\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT, APPRNPAPI._CS_KS5601);		// Korean
            //APPRNPAPI.PrintTextW("你好\n\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT, APPRNPAPI._CS_BIG5);				// Chinese (BIG5)
            //APPRNPAPI.PrintTextW("你好\n\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT, APPRNPAPI._CS_GB2312);			// Chinese (gb2312)
            //APPRNPAPI.PrintTextW("こんにちは\n\n", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT, APPRNPAPI._CS_SHIFT_JIS);	// Shift_JIS

            APPRNPAPI.CutPaper();
        }

        private void btnPrintBitmap_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileOpen     = new OpenFileDialog();
            fileOpen.DefaultExt         = "";
            fileOpen.InitialDirectory   = "";
            fileOpen.Filter             = "Image File(*.bmp)|*.bmp|Image File(*.jpg)|*.jpg|Image File(*.gif)|*.gif||";
            fileOpen.FilterIndex        = 1;
            fileOpen.RestoreDirectory   = true;

            if (fileOpen.ShowDialog() != DialogResult.OK)
                return;

            APPRNPAPI.PrintBitmap(Encoding.Default.GetBytes(fileOpen.FileName), APPRNPAPI._WIDTH_NONE, APPRNPAPI._ALIGNMENT_CENTER, 50, true);
            APPRNPAPI.CutPaper();
        }

        private void btn1DBarcode_Click(object sender, EventArgs e)
        {
            //UPC-A
            APPRNPAPI.PrintText("BARCODE : UPCA\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("075678164125"), APPRNPAPI._BCS_UPCA, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            //UPC-E
            APPRNPAPI.PrintText("BARCODE : UPCE\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("042100005264"), APPRNPAPI._BCS_UPCE, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            //JAN,EAN8	
            APPRNPAPI.PrintText("BARCODE : JAN/EAN8\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("1234567"), APPRNPAPI._BCS_JAN8, 50, 2, APPRNPAPI._ALIGNMENT_RIGHT, APPRNPAPI._BC_TEXT_ABOVE);
            APPRNPAPI.LineFeed(2);


            //JAN,EAN13	
            APPRNPAPI.PrintText("BARCODE : JAN/EAN13\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("123456789012"), APPRNPAPI._BCS_EAN13, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            // Code39
            APPRNPAPI.PrintText("BARCODE : CODE39\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("12"), APPRNPAPI._BCS_CODE39, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            // ITF
            APPRNPAPI.PrintText("BARCODE : ITF\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("1234"), APPRNPAPI._BCS_ITF, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            // CODABAR
            APPRNPAPI.PrintText("BARCODE : CODABAR\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("1234"), APPRNPAPI._BCS_CODABAR, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            // CODE3
            APPRNPAPI.PrintText("BARCODE : CODE93\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("1234"), APPRNPAPI._BCS_CODE93, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.LineFeed(2);

            // CODE128
            APPRNPAPI.PrintText("BARCODE : CODE128\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("1234"), APPRNPAPI._BCS_CODE128, 50, 2, APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.CutPaper();
        }

        private void btn2DBarcode_Click(object sender, EventArgs e)
        {
            APPRNPAPI.PrintText("PDF417\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintPDF417(Encoding.Default.GetBytes("12345647890123456478901234564789012345647890"), APPRNPAPI._PDF417_TYPE1, 0, 0, 3, 2, APPRNPAPI._PDF417_ECC_LEVEL_0, APPRNPAPI._ALIGNMENT_LEFT);
            APPRNPAPI.LineFeed(2);

            APPRNPAPI.PrintText("QR Code\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintQRCode(Encoding.Default.GetBytes("12345647890123456478901234564789012345647890"), APPRNPAPI._QRCODE_MODEL1, 4, APPRNPAPI._QRCODE_ECC_LEVEL_L, APPRNPAPI._ALIGNMENT_LEFT);
            APPRNPAPI.LineFeed(2);

            APPRNPAPI.PrintText("DataMatrix\n", APPRNPAPI._ALIGNMENT_LEFT, 0, 0);
            APPRNPAPI.PrintDataMatrix(Encoding.Default.GetBytes("12345647890123456478901234564789012345647890"), 3, APPRNPAPI._ALIGNMENT_LEFT);
            APPRNPAPI.LineFeed(2);

            APPRNPAPI.CutPaper();
        }

        private void btnDirectIO_Click(object sender, EventArgs e)
        {
            string strEr;
            string strRet;

            Byte dbRead = 0x00;
	        uint dwRead = 1;

            int lResult = APPRNPAPI.DirectIO(new byte[] { 0x10, 0x04, 0x02 }, 3, ref dbRead, ref dwRead);

            if (lResult != APPRNPAPI._SUCCESS){
		        strEr = "Error Code = " + lResult;
		        MessageBox.Show(strEr);		
		        return;
	        }
	        else{
                strRet = String.Format("Read Data : 0x{0:X}", dbRead);
		        MessageBox.Show(strRet);
	        }
        }

        private void btnStatus_Click(object sender, EventArgs e)
        {
            string strStatus = "Printer Status = ";
            int lState = (int)APPRNPAPI.GetPrinterCurrentStatus();

            if ((lState & APPRNPAPI._STS_PAPEREMPTY) == APPRNPAPI._STS_PAPEREMPTY)              strStatus += " PAPER EMPTY ";
            if ((lState & APPRNPAPI._STS_CASHDRAWER_HIGH) == APPRNPAPI._STS_CASHDRAWER_HIGH)    strStatus += " CASHDRAWER HIGH ";
            if ((lState & APPRNPAPI._STS_CASHDRAWER_LOW) == APPRNPAPI._STS_CASHDRAWER_LOW)      strStatus += " CASHDRAWER LOW ";
            if ((lState & APPRNPAPI._STS_COVEROPEN) == APPRNPAPI._STS_COVEROPEN)                strStatus += " COVER OPEN ";
            if ((lState & APPRNPAPI._STS_ERROR) == APPRNPAPI._STS_ERROR){
                strStatus += " OFFLINE ";
            }

            if (lState == APPRNPAPI._STS_NORMAL)                                                
                strStatus += " NORMAL ";

            MessageBox.Show(strStatus);
        }

        private void btnPrintSample_Click(object sender, EventArgs e)
        {
            APPRNPAPI.InitializePrinter();
            APPRNPAPI.SetCharacterSet(APPRNPAPI._CS_WPC1252);
            APPRNPAPI.SetInterChrSet(APPRNPAPI._ICS_USA);

	        APPRNPAPI.PrintText("Receipt", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD | APPRNPAPI._FT_UNDERLINE, APPRNPAPI._TS_1WIDTH | APPRNPAPI._TS_1HEIGHT);	
	        APPRNPAPI.PrintText(" \n\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD | APPRNPAPI._FT_UNDERLINE, APPRNPAPI._TS_1WIDTH | APPRNPAPI._TS_1HEIGHT);	

	        APPRNPAPI.PrintText("------------------------------\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
	        APPRNPAPI.PrintText(" Apple      3             $3.0\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
	        APPRNPAPI.PrintText(" Cola       1             $1.2\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText(" Milk       2             $1.8\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText(" Juice      1             $2.0\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("------------------------------\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText(" Total                    $8.0\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText(" Cash                     $8.0\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("                          $0.0\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	

	        APPRNPAPI.PrintText("Test", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("BBB", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_FONTB | APPRNPAPI._FT_BOLD, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("CCC", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_FONTB | APPRNPAPI._FT_REVERSE, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("DDD", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD | APPRNPAPI._FT_UNDERLINE, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintText("\n", APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._FT_BOLD | APPRNPAPI._FT_UNDERLINE, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);	
	        APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("12345678"), APPRNPAPI._BCS_ITF, 100, 2, APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._BC_TEXT_BELOW);
            APPRNPAPI.CutPaper();
        }

        public int statusCallBackMethod(int nStatus)
        {
            return interpretPrinterStatusData(nStatus);
        }

        public int interpretPrinterStatusData(int status)
        {
            statusTextBox.AppendText(DateTime.Now + " : " + "\r\n");

            if ((status & APPRNPAPI._STS_PAPEREMPTY) == APPRNPAPI._STS_PAPEREMPTY) 
                statusTextBox.AppendText(" - PAPER-EMPTY" + "\r\n");
            if ((status & APPRNPAPI._STS_CASHDRAWER_HIGH) == APPRNPAPI._STS_CASHDRAWER_HIGH)
                statusTextBox.AppendText(" - CASHDRAWER-HIGH" + "\r\n");
            if ((status & APPRNPAPI._STS_CASHDRAWER_LOW) == APPRNPAPI._STS_CASHDRAWER_LOW)
                statusTextBox.AppendText(" - CASHDRAWER-LOW" + "\r\n");
            if ((status & APPRNPAPI._STS_COVEROPEN) == APPRNPAPI._STS_COVEROPEN)
                statusTextBox.AppendText(" - COVER-OPEN" + "\r\n");
            if ((status & APPRNPAPI._STS_ERROR) == APPRNPAPI._STS_ERROR)
                statusTextBox.AppendText(" - OFFLINE" + "\r\n");
            if (status == APPRNPAPI._STS_NORMAL)
                statusTextBox.AppendText(" - NORMAL" + "\r\n");
            return 0;
        }

        private void btnStatus_Set_Click(object sender, EventArgs e)
        {
            statusCallBackDelegate = new APPRNPAPI.CallBackDelegate(statusCallBackMethod);
            if (APPRNPAPI.BidiSetCallBack(statusCallBackDelegate) != APPRNPAPI._SUCCESS){
                statusCallBackDelegate = null;
            }
            else{
                btnStatus_Set.Enabled = false;
                btnStatus_Cancel.Enabled = true;
            }
        }

        private void btnStatus_Cancel_Click(object sender, EventArgs e)
        {
            APPRNPAPI.BidiCancelCallBack();
            statusCallBackDelegate = null;
            btnStatus_Set.Enabled = true;
            btnStatus_Cancel.Enabled = false;
            statusTextBox.Clear();
        }

        private void btnPmDirection_Click(object sender, EventArgs e)
        {
            APPRNPAPI.InitializePrinter();
            APPRNPAPI.SetPagemode(APPRNPAPI._PRINTER_PAGEMODE_IN /* PAGE MODE IN */);
            APPRNPAPI.SetPagemodePrintArea(0, 0, 512, 600);
            APPRNPAPI.SetPagemodeDirection(APPRNPAPI._PRINTER_PD_0);
            APPRNPAPI.PrintText("Left to right", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.SetPagemodeDirection(APPRNPAPI._PRINTER_PD_LEFT90);
            APPRNPAPI.PrintText("Bottom to Top", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.SetPagemodeDirection(APPRNPAPI._PRINTER_PD_180);
            APPRNPAPI.PrintText("Right to Left", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.SetPagemodeDirection(APPRNPAPI._PRINTER_PD_RIGHT90);
            APPRNPAPI.PrintText("Top to Bottom", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            APPRNPAPI.SetPagemode(APPRNPAPI._PRINTER_PAGEMODE_OUT /* PAGE MODE OUT */);
            APPRNPAPI.CutPaper();
        }

        private void btnPmSample_Click(object sender, EventArgs e)
        {
            int xPosition = 50;
            int yPosition = 50;
            APPRNPAPI.InitializePrinter();
            APPRNPAPI.SetPagemode(APPRNPAPI._PRINTER_PAGEMODE_IN /* PAGE MODE IN */);
            APPRNPAPI.SetPagemodePrintArea(0, 0, 512, 600);
            APPRNPAPI.SetPagemodeDirection(APPRNPAPI._PRINTER_PD_RIGHT90 /* TOP TO BOTTOM*/);
            APPRNPAPI.SetPagemodePosition(xPosition, yPosition);
            APPRNPAPI.PrintText("Sample Text", APPRNPAPI._ALIGNMENT_LEFT, APPRNPAPI._FT_DEFAULT, APPRNPAPI._TS_0WIDTH | APPRNPAPI._TS_0HEIGHT);
            yPosition += 70;
            APPRNPAPI.SetPagemodePosition(xPosition, yPosition);
            APPRNPAPI.PrintBarcode(Encoding.Default.GetBytes("12345678"), APPRNPAPI._BCS_CODE128, 100, 2, APPRNPAPI._ALIGNMENT_CENTER, APPRNPAPI._BC_TEXT_NONE);
            APPRNPAPI.SetPagemode(APPRNPAPI._PRINTER_PAGEMODE_OUT /* PAGE MODE OUT */);
            APPRNPAPI.CutPaper();
        }
    }
}